@@ -413,22 +413,32 @@ void KDirListerTest::testNewItemsInSymlink() // #213799
         qDebug() << "Deleting" << (path + fileName);
         QTest::qWait(1000); // for timestamp difference
         QFile::remove(path + fileName);
-        QTRY_COMPARE(dirLister2.spyItemsDeleted.count(), 1);
-        QTRY_COMPARE(m_dirLister.spyItemsDeleted.count(), 1);
+        QTRY_COMPARE_WITH_TIMEOUT(dirLister2.spyItemsDeleted.count(), 1, 1000);
+        QTRY_COMPARE_WITH_TIMEOUT(m_dirLister.spyItemsDeleted.count(), 1, 1000);
         const KFileItem item = dirLister2.spyItemsDeleted[0][0].value<KFileItemList>().at(0);
         QCOMPARE(item.url().toLocalFile(), QString(symPath + '/' + fileName));
+
+        dirLister2.spyItemsDeleted.clear();
+        m_dirLister.spyItemsDeleted.clear();
     }
     // Test file deletion in symlink dir #469254
     {
         qDebug() << "Deleting" << (symPath + "/" + fileName2);
         QTest::qWait(1000); // for timestamp difference
         QFile::remove(symPath + "/" + fileName2);
-        QTRY_COMPARE(dirLister2.spyItemsDeleted.count(), 2);
-        QTRY_COMPARE(m_dirLister.spyItemsDeleted.count(), 2);
-        const KFileItem item = dirLister2.spyItemsDeleted[1][0].value<KFileItemList>().at(0);
-        QCOMPARE(item.url().toLocalFile(), QString(symPath + '/' + fileName2));
+
+        QTRY_COMPARE_WITH_TIMEOUT(m_dirLister.spyItemsDeleted.count(), 1, 1000);
+        const KFileItem item = m_dirLister.spyItemsDeleted[0][0].value<KFileItemList>().at(0);
+        QCOMPARE(item.url().toLocalFile(), QString(path + fileName2));
+
+        QTRY_COMPARE_WITH_TIMEOUT(dirLister2.spyItemsDeleted.count(), 1, 1000);
+        const KFileItem item2 = dirLister2.spyItemsDeleted[0][0].value<KFileItemList>().at(0);
+        QCOMPARE(item2.url().toLocalFile(), QString(symPath + '/' + fileName2));
     }
     QFile::remove(symPath);
+
+    m_dirLister.spyItemsDeleted.clear();
+    dirLister2.spyItemsDeleted.clear();
 }
 
 // This test assumes testOpenUrl was run before. So m_dirLister is holding the items already.
@@ -1446,16 +1456,20 @@ void KDirListerTest::testRenameDirectory() // #401552
         slotOpenUrlOnRename(newUrl);
     });
 
-    for (int i = 0; i < dirs.size(); i++) {
+    for (const auto &newDir : dirs) {
         // Wait for the listener to get all files
         QTRY_VERIFY(m_dirLister.isFinished());
         // Do the rename
-        QString newDir = dirs.at(i);
         job = KIO::rename(QUrl::fromLocalFile(currDir), QUrl::fromLocalFile(newDir), KIO::HideProgressInfo);
         QVERIFY2(job->exec(), qPrintable(job->errorString()));
         QTest::qWait(500); // Without the delay the crash doesn't happen
         currDir = newDir;
     }
+
+    // cleanup
+    const auto delJob = KIO::del(QUrl::fromLocalFile(dirs.last()));
+    QVERIFY(delJob->exec());
+
     disconnect(&m_dirLister, nullptr, this, nullptr);
 }
 
@@ -1539,8 +1553,7 @@ void KDirListerTest::testMimeFilter()
     lister.setMimeFilter(mimeTypes);
     lister.openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags);
 
-    QSignalSpy spyCompleted(&lister, qOverload<>(&KCoreDirLister::completed));
-    QVERIFY(spyCompleted.wait(1000));
+    QVERIFY(lister.spyCompleted.wait(1000));
 
     QCOMPARE(lister.items().size(), filteredFiles.size());
 
@@ -1554,6 +1567,9 @@ void KDirListerTest::testDeleteCurrentDir()
 {
     // ensure m_dirLister holds the items.
     m_dirLister.openUrl(QUrl::fromLocalFile(tempPath()), KDirLister::NoFlags);
+
+    QVERIFY(m_dirLister.spyCompleted.wait(1000));
+
     m_dirLister.clearSpies();
     KIO::DeleteJob *job = KIO::del(QUrl::fromLocalFile(tempPath()), KIO::HideProgressInfo);
     bool ok = job->exec();
@@ -1562,14 +1578,12 @@ void KDirListerTest::testDeleteCurrentDir()
 
     QCOMPARE(m_dirLister.spyClearDir.count(), 0);
 
-    QList<QUrl> deletedUrls;
-    for (int i = 0; i < m_dirLister.spyItemsDeleted.count(); ++i) {
-        deletedUrls += m_dirLister.spyItemsDeleted[i][0].value<KFileItemList>().urlList();
+    // there can be duplicated delete events
+    QVERIFY(m_dirLister.spyItemsDeleted.count() >= 1 && m_dirLister.spyItemsDeleted.count() <= 2);
+    const QUrl currentDirUrl = QUrl::fromLocalFile(tempPath()).adjusted(QUrl::StripTrailingSlash);
+    for (const auto &deletedItem : m_dirLister.spyItemsDeleted) {
+        QCOMPARE(currentDirUrl, deletedItem.at(0).value<KFileItemList>().at(0).url());
     }
-    // qDebug() << deletedUrls;
-    QUrl currentDirUrl = QUrl::fromLocalFile(tempPath()).adjusted(QUrl::StripTrailingSlash);
-    // Sometimes I get ("current/subdir", "current") here, but that seems ok.
-    QVERIFY(deletedUrls.contains(currentDirUrl));
 }
 
 void KDirListerTest::testForgetDir()
@@ -1707,4 +1721,78 @@ void KDirListerTest::testCacheEviction()
     QVERIFY(KDirWatch::self()->contains(newDir.path() + QString("dir_1")));
 }
 
+void KDirListerTest::testUnreadableParentDirectory() {
+#ifdef WITH_QTDBUS
+    QTemporaryDir newDir(tmpDirTemplate());
+    MyDirLister dirLister;
+
+    const QString hiddenPath = newDir.path() + QString("/hidden");
+    const QString visiblePath = hiddenPath + QString("/visible");
+
+    KDirWatch::self()->addDir(newDir.path(), KDirWatch::WatchSubDirs);
+
+    // Create hidden folder and add it to lister
+    QVERIFY(QDir().mkdir(hiddenPath));
+    // Set folder to u-r to mimic `chown root:root hidden`
+    const mode_t badperms = (S_IWUSR | S_IXUSR | S_IXGRP);
+    QVERIFY(KIO::chmod(QUrl::fromLocalFile(hiddenPath), badperms)->exec());
+    // Set the permissions normal to allow test to clean up at end of scope
+    auto cleanup = qScopeGuard([hiddenPath] {
+        const mode_t clearperms = (S_IWUSR | S_IXUSR | S_IRUSR | S_IRGRP | S_IWGRP | S_IXGRP);
+        QVERIFY(KIO::chmod(QUrl::fromLocalFile(hiddenPath), clearperms)->exec());
+    });
+    QVERIFY(dirLister.openUrl(QUrl::fromLocalFile(hiddenPath), MyDirLister::Keep));
+    // This should fail since we cant read the folder, so check for jobError
+    QVERIFY(dirLister.spyJobError.wait(500));
+    QVERIFY(dirLister.isFinished());
+    QVERIFY(KDirWatch::self()->contains(hiddenPath));
+
+    // Create visible folder and add it to lister
+    QVERIFY(QDir().mkdir(visiblePath));
+    QVERIFY(dirLister.openUrl(QUrl::fromLocalFile(visiblePath), MyDirLister::Keep));
+    QVERIFY(dirLister.spyCompleted.wait(500));
+    QVERIFY(dirLister.isFinished());
+    QVERIFY(KDirWatch::self()->contains(visiblePath));
+
+    // Wait until the time changes so the cache will have to be updated
+    waitUntilMTimeChange(hiddenPath);
+    const QString hiddenFile = hiddenPath + "/aaaa";
+    createSimpleFile(hiddenFile);
+
+    // Add a file to have difference between old and new cache data
+    waitUntilMTimeChange(visiblePath);
+    const QString visibleFile = visiblePath + "/bbbb";
+    createSimpleFile(visibleFile);
+    // Make sure we emit files changed so the cache will be re-read
+    org::kde::KDirNotify::emitFilesChanged(QList<QUrl>{QUrl::fromLocalFile(visiblePath), QUrl::fromLocalFile(hiddenPath)});
+#endif
+}
+
+void KDirListerTest::testPathWithSquareBrackets()
+{
+#if QT_VERSION == QT_VERSION_CHECK(6, 8, 3) || QT_VERSION == QT_VERSION_CHECK(6, 9, 0) || QT_VERSION == QT_VERSION_CHECK(6, 10, 0)
+    QSKIP("This test is expected to fail on Qt 6.8.3 / 6.9.0 / wanna-be-6.10.0 ");
+#endif
+    QTemporaryDir newDir(tmpDirTemplate());
+    QFile file(newDir.filePath("[test].txt"));
+    QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
+    file.close();
+    MyDirLister dirLister;
+    dirLister.openUrl(QUrl::fromLocalFile(newDir.path()));
+
+    QVERIFY(dirLister.spyCompleted.wait(500));
+    QVERIFY(dirLister.isFinished());
+
+    m_refreshedItems.clear();
+    connect(&dirLister, &KCoreDirLister::refreshItems, this, &KDirListerTest::slotRefreshItems);
+    QSignalSpy spyRefreshItems(&dirLister, &KCoreDirLister::refreshItems);
+
+    QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
+    file.write(QByteArray("foo"));
+    file.close();
+    QVERIFY(spyRefreshItems.wait(1000));
+    QCOMPARE(m_refreshedItems.count(), 1);
+    QCOMPARE(m_refreshedItems.at(0).first.url(), QUrl::fromLocalFile(file.fileName()));
+}
+
 #include "moc_kdirlistertest.cpp"
