@@ -17,6 +17,7 @@
 #include "kdiroperatoriconview_p.h"
 #include "kdirsortfilterproxymodel.h"
 #include "kfileitem.h"
+#include "kfileitemselectionemblem.h"
 #include "kfilemetapreview_p.h"
 #include "knewfilemenu.h"
 #include "kpreviewwidgetbase.h"
@@ -34,7 +35,8 @@
 #include <KMessageBox>
 #include <KProtocolManager>
 #include <KSharedConfig>
-#include <KStandardAction>
+#include <KStandardActions>
+#include <KToggleAction>
 #include <KUrlMimeData>
 #include <kfileitemdelegate.h>
 #include <kfilepreviewgenerator.h>
@@ -55,6 +57,7 @@
 #include <QProgressBar>
 #include <QRegularExpression>
 #include <QScrollBar>
+#include <QScroller>
 #include <QSplitter>
 #include <QStack>
 #include <QTimer>
@@ -216,6 +219,9 @@ public:
     bool m_shouldFetchForItems = false;
     bool m_isSaving = false;
     bool m_showOpenWithActions = false;
+    bool m_isTouchEvent = false;
+    bool m_isTouchDrag = false;
+    bool m_keyNavigation = false;
 
     QList<QUrl> m_itemsToBeSetAsCurrent;
     QStringList m_supportedSchemes;
@@ -1239,6 +1245,11 @@ void KDirOperator::showOpenWithActions(bool enable)
     d->m_showOpenWithActions = enable;
 }
 
+bool KDirOperator::usingKeyNavigation()
+{
+    return d->m_keyNavigation;
+}
+
 void KDirOperator::changeEvent(QEvent *event)
 {
     QWidget::changeEvent(event);
@@ -1249,10 +1260,20 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
     // If we are not hovering any items, check if there is a current index
     // set. In that case, we show the preview of that item.
     switch (event->type()) {
+    case QEvent::TouchBegin:
+        d->m_isTouchEvent = true;
+        break;
     case QEvent::MouseMove: {
-        if (d->m_preview && !d->m_preview->isHidden()) {
-            const QModelIndex hoveredIndex = d->m_itemView->indexAt(d->m_itemView->viewport()->mapFromGlobal(QCursor::pos()));
+        if (d->m_isTouchEvent) {
+            return true;
+        }
 
+        const QModelIndex hoveredIndex = d->m_itemView->indexAt(d->m_itemView->viewport()->mapFromGlobal(QCursor::pos()));
+        if (hoveredIndex.isValid()) {
+            KFileItemSelectionEmblem(d->m_itemView, hoveredIndex, this).updateSelectionEmblemRectForIndex(iconSize());
+        }
+
+        if (d->m_preview && !d->m_preview->isHidden()) {
             if (d->m_lastHoveredIndex == hoveredIndex) {
                 return QWidget::eventFilter(watched, event);
             }
@@ -1270,6 +1291,24 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
         }
         break;
     }
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseButtonDblClick: {
+        QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
+        if (mouseEvent) {
+            // Navigate and then discard the event so it doesn't select an item on the directory navigated to.
+            switch (mouseEvent->button()) {
+            case Qt::BackButton:
+                back();
+                return true;
+            case Qt::ForwardButton:
+                forward();
+                return true;
+            default:
+                break;
+            }
+        }
+        break;
+    }
     case QEvent::MouseButtonRelease: {
         if (d->m_preview != nullptr && !d->m_preview->isHidden()) {
             const QModelIndex hoveredIndex = d->m_itemView->indexAt(d->m_itemView->viewport()->mapFromGlobal(QCursor::pos()));
@@ -1279,17 +1318,24 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
                 d->m_preview->clearPreview();
             }
         }
-        QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
-        if (mouseEvent) {
-            switch (mouseEvent->button()) {
-            case Qt::BackButton:
-                back();
-                break;
-            case Qt::ForwardButton:
-                forward();
-                break;
-            default:
-                break;
+
+        d->m_isTouchEvent = false;
+        if (d->m_isTouchDrag) {
+            d->m_isTouchDrag = false;
+            return true;
+        }
+
+        break;
+    }
+    case QEvent::HoverLeave: {
+        if (d->m_preview && !d->m_preview->isHidden()) {
+            const QModelIndex currentIndex = d->m_itemView->selectionModel() ? d->m_itemView->selectionModel()->currentIndex() : QModelIndex();
+
+            if (currentIndex.isValid() && d->m_itemView->selectionModel()->isSelected(currentIndex)) {
+                const KFileItem item = d->m_itemView->model()->data(currentIndex, KDirModel::FileItemRole).value<KFileItem>();
+                if (!item.isNull()) {
+                    d->m_preview->showPreview(item.url());
+                }
             }
         }
         break;
@@ -1389,6 +1435,19 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
                 return true;
             }
         }
+        // Only use tab key to escape the view navigation
+        if (evt->key() == Qt::Key_Tab) {
+            d->m_keyNavigation = false;
+            d->slotSelectionChanged();
+            // When saving we need to return here,
+            // otherwise we skip over the next item with our tab press
+            // since we focus on that item in slotSelectionChanged
+            if (d->m_isSaving) {
+                return true;
+            }
+        } else {
+            d->m_keyNavigation = true;
+        }
         break;
     }
     case QEvent::Resize: {
@@ -1475,11 +1534,11 @@ QAbstractItemView *KDirOperator::createView(QWidget *parent, KFile::FileView vie
 {
     QAbstractItemView *itemView = nullptr;
     if (KFile::isDetailView(viewKind) || KFile::isTreeView(viewKind) || KFile::isDetailTreeView(viewKind)) {
-        KDirOperatorDetailView *detailView = new KDirOperatorDetailView(parent);
+        KDirOperatorDetailView *detailView = new KDirOperatorDetailView(this, parent);
         detailView->setViewMode(viewKind);
         itemView = detailView;
     } else {
-        itemView = new KDirOperatorIconView(parent, decorationPosition());
+        itemView = new KDirOperatorIconView(this, parent, decorationPosition());
     }
 
     return itemView;
@@ -1589,6 +1648,24 @@ void KDirOperator::setViewInternal(QAbstractItemView *view)
     d->m_pendingMimeTypes.clear();
     const bool listDir = (d->m_itemView == nullptr);
 
+    if (d->m_itemView) {
+        d->m_itemView->viewport()->removeEventFilter(this);
+    }
+    QScroller *scroller = QScroller::scroller(view->viewport());
+    QScrollerProperties scrollerProp;
+    scrollerProp.setScrollMetric(QScrollerProperties::AcceleratingFlickMaximumTime, 0.2); // QTBUG-88249
+    scroller->setScrollerProperties(scrollerProp);
+    scroller->grabGesture(view->viewport());
+    connect(scroller, &QScroller::stateChanged, this, [this](const QScroller::State newState) {
+        if (newState == QScroller::Inactive) {
+            d->m_isTouchEvent = false;
+        } else if (newState == QScroller::Dragging) {
+            d->m_isTouchDrag = true;
+        }
+    });
+    view->viewport()->grabGesture(Qt::TapGesture);
+    view->setAttribute(Qt::WA_AcceptTouchEvents);
+
     if (d->m_mode & KFile::Files) {
         view->setSelectionMode(QAbstractItemView::ExtendedSelection);
     } else {
@@ -1682,8 +1759,8 @@ void KDirOperator::setViewInternal(QAbstractItemView *view)
 
     const bool previewForcedToTrue = d->m_inlinePreviewState == KDirOperatorPrivate::ForcedToTrue;
     const bool previewShown = d->m_inlinePreviewState == KDirOperatorPrivate::NotForced ? d->m_showPreviews : previewForcedToTrue;
-    d->m_previewGenerator = new KFilePreviewGenerator(d->m_itemView);
     d->m_itemView->setIconSize(previewForcedToTrue ? QSize(KIconLoader::SizeHuge, KIconLoader::SizeHuge) : QSize(d->m_iconSize, d->m_iconSize));
+    d->m_previewGenerator = new KFilePreviewGenerator(d->m_itemView);
     d->m_previewGenerator->setPreviewShown(previewShown);
     action(KDirOperator::ShowPreview)->setChecked(previewShown);
 
@@ -1775,6 +1852,7 @@ void KDirOperator::selectFile(const KFileItem &item)
     QApplication::restoreOverrideCursor();
 
     Q_EMIT fileSelected(item);
+    d->m_keyNavigation = false;
 }
 
 void KDirOperator::highlightFile(const KFileItem &item)
@@ -1784,6 +1862,7 @@ void KDirOperator::highlightFile(const KFileItem &item)
     }
 
     Q_EMIT fileHighlighted(item);
+    d->m_keyNavigation = false;
 }
 
 void KDirOperator::setCurrentItem(const QUrl &url)
@@ -1930,26 +2009,26 @@ void KDirOperator::setupActions()
     d->m_actionMenu = new KActionMenu(i18n("Menu"), this);
     d->m_actions[PopupMenu] = d->m_actionMenu;
 
-    QAction *upAction = KStandardAction::create(KStandardAction::Up, this, SLOT(cdUp()), this);
+    QAction *upAction = KStandardActions::create(KStandardActions::Up, this, &KDirOperator::cdUp, this);
     d->m_actions[Up] = upAction;
     upAction->setText(i18n("Parent Folder"));
 
-    QAction *backAction = KStandardAction::create(KStandardAction::Back, this, SLOT(back()), this);
+    QAction *backAction = KStandardActions::create(KStandardActions::Back, this, &KDirOperator::back, this);
     d->m_actions[Back] = backAction;
     auto backShortcuts = backAction->shortcuts();
     backShortcuts << Qt::Key_Backspace;
     backAction->setShortcuts(backShortcuts);
     backAction->setToolTip(i18nc("@info", "Go back"));
 
-    QAction *forwardAction = KStandardAction::create(KStandardAction::Forward, this, SLOT(forward()), this);
+    QAction *forwardAction = KStandardActions::create(KStandardActions::Forward, this, &KDirOperator::forward, this);
     d->m_actions[Forward] = forwardAction;
     forwardAction->setToolTip(i18nc("@info", "Go forward"));
 
-    QAction *homeAction = KStandardAction::create(KStandardAction::Home, this, SLOT(home()), this);
+    QAction *homeAction = KStandardActions::create(KStandardActions::Home, this, &KDirOperator::home, this);
     d->m_actions[Home] = homeAction;
     homeAction->setText(i18n("Home Folder"));
 
-    QAction *reloadAction = KStandardAction::create(KStandardAction::Redisplay, this, SLOT(rereadDir()), this);
+    QAction *reloadAction = KStandardActions::create(KStandardActions::Redisplay, this, &KDirOperator::rereadDir, this);
     d->m_actions[Reload] = reloadAction;
     reloadAction->setText(i18n("Reload"));
     reloadAction->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Reload));
@@ -1961,7 +2040,7 @@ void KDirOperator::setupActions()
         mkdir();
     });
 
-    QAction *rename = KStandardAction::renameFile(this, &KDirOperator::renameSelected, this);
+    QAction *rename = KStandardActions::renameFile(this, &KDirOperator::renameSelected, this);
     d->m_actions[Rename] = rename;
 
     QAction *trash = new QAction(i18n("Move to Trash"), this);
